/*****************************************************************************/

/*
 *	cpu.c -- simple CPU usage reporting tool.
 *
 *	(C) Copyright 2000, Greg Ungerer (gerg@snapgear.com)
 */

/*****************************************************************************/

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>
#include <getopt.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include "adslif.h"

typedef struct CMD_TYPE {
  char *cmd;
  int setFlag;  //=2, use cmd arg to set DMT, =1, no arg set
  int argsize;
  int funcID;
} cmd_t;

cmd_t cmdTable[]={
{"startAdsl", 		2,	4, 	RLCM_PHY_START_MODEM},
{"EnableModemLine", 	1,	0, 	RLCM_PHY_ENABLE_MODEM},
{"DisableModemLine", 	1,	0, 	RLCM_PHY_DISABLE_MODEM},
{"GetDriverVersion", 	1,	0, 	RLCM_GET_DRIVER_VERSION},
{"GetDriverBuild", 	1,	0, 	RLCM_GET_DRIVER_BUILD},
{"ReTrain", 		1,	0, 	RLCM_MODEM_RETRAIN},
{"GetReHSKCount", 	1,	0, 	RLCM_GET_REHS_COUNT},
{"GetChannelSNR", 	1,	0, 	RLCM_GET_CHANNEL_SNR},
{"GetAverageSNR", 	1,	0, 	RLCM_GET_AVERAGE_SNR},
{"GetSNRMargin", 	1,	0, 	RLCM_GET_SNR_MARGIN},
{"ReportModemStates", 	1,	0, 	RLCM_REPORT_MODEM_STATE},
{"ReportPMdata", 	1,	0, 	RLCM_REPORT_PM_DATA},
{"NearEndIdentification", 	1,	0, 	RLCM_MODEM_NEAR_END_ID_REQ},
{"FarEndIdentification", 	1,	0, 	RLCM_MODEM_FAR_END_ID_REQ},
{"NearEndLineData", 	1,	0, 	RLCM_MODEM_NEAR_END_LINE_DATA_REQ},
{"FarEndLineData", 	1,	0, 	RLCM_MODEM_FAR_END_LINE_DATA_REQ},
{"NearEndChannelDataFastMode", 	1,	0, 	RLCM_MODEM_NEAR_END_FAST_CH_DATA_REQ},
{"NearEndChannelDataInterleavedMode", 	1,	0, 	RLCM_MODEM_NEAR_END_INT_CH_DATA_REQ},
{"FarEndChannelDataFastMode", 	1,	0, 	RLCM_MODEM_FAR_END_FAST_CH_DATA_REQ},
{"FarEndChannelDataInterleavedMode", 	1,	0, 	RLCM_MODEM_FAR_END_INT_CH_DATA_REQ},
{"SetAdslMode", 	2,	4, 	RLCM_SET_ADSL_MODE},
{"GetAdslMode", 	1,	0, 	RLCM_GET_ADSL_MODE},
{"GetLossData", 	1,	0, 	RLCM_GET_CURRENT_LOSS_DATA},
{"GetLinkSpeed", 	1,	0, 	RLCM_GET_LINK_SPEED},
{"GetChannelMode", 	1,	0, 	RLCM_GET_CHANNEL_MODE},
{"GetLoopAtt", 		1,	0, 	RLCM_GET_LOOP_ATT},
{"IncTxPower", 		2,	4, 	RLCM_INC_TX_POWER},
{"TunePerf", 		2,	4, 	RLCM_TUNE_PERF},
{"EnableBitSwap", 	1,	0, 	RLCM_ENABLE_BIT_SWAP},
{"DisableBitSwap", 	1,	0, 	RLCM_DISABLE_BIT_SWAP},
{"EnablePilotRelocation", 	1,	0, 	RLCM_ENABLE_PILOT_RELOCATION},
{"DisablePilotRelocation", 	1,	0, 	RLCM_DISABLE_PILOT_RELOCATION},
{"EnableTrellis", 	1,	0, 	RLCM_ENABLE_TRELLIS},
{"DisableTrellis", 	1,	0, 	RLCM_DISABLE_TRELLIS},
{"SetVendorID", 	2,	4, 	RLCM_SET_VENDOR_ID},
{"ReadCfg", 		1,	0, 	RLCM_MODEM_READ_CONFIG},
//{"WriteCfg", 		2,	RLCM_MODEM_CONFIG_SIZE, 	RLCM_MODEM_WRITE_CONFIG},
{"DebugMode", 		2,	4, 	RLCM_DEBUG_MODE},
{"TestPSD", 		2,	4, 	RLCM_TEST_PSD},
{"GetChannelBH",	1,	0, 	RLCM_GET_CHANNEL_BH},
{"msgmode",		2,	4, 	RLCM_MSGMODE},
{"reset",		2,	4, 	RLCM_SET_ADSL_MODE},
//tylo, for 8671
{"SendDyingGasp",	1,	0,	RLCM_SEND_DYING_GASP},
{"TestHW",			1,	0,	RLCM_TEST_HW},
{"SetLoopback",		1,	0,	RLCM_SET_LOOPBACK},
{"InitAdslMode",		2,	4,	RLCM_INIT_ADSL_MODE},
{"EnablePOMAccess",	1,	0,	RLCM_ENABLE_POM_ACCESS},
{"DisablePOMAccess",	1,	0,	RLCM_DISABLE_POM_ACCESS},
{"MaskTones",		2,	8,	RLCM_MASK_TONE},//masktones
{"GetCapability",		2,	4,	RLCM_GET_CAPABILITY},
{"VerifyHW",			2,	4,	RLCM_VERIFY_HW},
{"TrigOlrType",		2,	4,	RLCM_TRIG_OLR_TYPE},
{"TrigOlrType1",		2,	4,	RLCM_TRIG_OLR_TYPE1},
{"EnableAEQ",			2,	4,	RLCM_ENABLE_AEQ},
{"EnableHPF",			2,	4,	RLCM_ENABLE_HPF},
{"SetHPFfc",			2,	4,	RLCM_SET_HPF_FC},
{"SetHybrid",			2,	4,	RLCM_SET_HYBRID},
{"SetRxGain",			2,	4,	RLCM_SET_RX_GAIN},
{"SetAFEReg",			2,	4,	RLCM_SET_AFE_REG},
{"SetFOBase",			2,	4,	RLCM_SET_FOBASE},
//yaru + for web data
{"SetXdslMode"		   ,2,4,        RLCM_SET_XDSL_MODE	},        	
{"GetShowtimeXdslMode"     ,1,0,        RLCM_GET_SHOWTIME_XDSL_MODE  },   
{"GetXdslMode"             ,1,0,        RLCM_GET_XDSL_MODE	        },
{"SetOLRType"              ,2,4,        RLCM_SET_OLR_TYPE		},
{"GetOLRType"              ,1,0,        RLCM_GET_OLR_TYPE 	        },
{"GetLineRate"             ,1,0,        RLCM_GET_LINE_RATE	        },
{"GetDsErrorCount"         ,1,0,        RLCM_GET_DS_ERROR_COUNT	},
{"GetUsErrorCount"         ,1,0,        RLCM_GET_US_ERROR_COUNT	},
{"GetDiagQLN"              ,1,0,        RLCM_GET_DIAG_QLN		},
{"GetDiagHlog"             ,1,0,        RLCM_GET_DIAG_HLOG	        },
{"GetDiagSNR"              ,1,0,        RLCM_GET_DIAG_SNR		},
{"GetDsPMSParam1"          ,1,0,        RLCM_GET_DS_PMS_PARAM1	},        
{"GetUsPMSParam1"          ,1,0,        RLCM_GET_US_PMS_PARAM1	},        
{"SetAnnexL"               ,2,4,        RLCM_SET_ANNEX_L		},
{"GetAnnexL"               ,1,0,        RLCM_GET_ANNEX_L		},
{"SetAnnexM"               ,2,4,        RLCM_SET_ANNEX_M		},
{"GetAnnexM"               ,1,0,        RLCM_GET_ANNEX_M		},
{"GetLinkPowerState"       ,1,0,        RLCM_GET_LINK_POWER_STATE	},
{"GetAttainableRate"       ,1,0,        RLCM_GET_GET_ATT_RATE	},
{"LoadCarrierMask"         ,1,0,        RLCM_LOADCARRIERMASK	},
{"Set8671Rev"              ,2,1,        RLCM_SET_8671_REV		},
{"Get8671Rev"              ,1,0,        RLCM_GET_8671_REV		},
{"SetHighInp"              ,2,4,        RLCM_SET_HIGH_INP		},
{"GetHighInp"              ,1,0,        RLCM_GET_HIGH_INP		},
//-yaru
//TR069
{"GetDslShowtime"	,1,0, 	RLCM_GET_DSL_STAT_SHOWTIME},
{"GetDslTotal" 		,1,0,	RLCM_GET_DSL_STAT_TOTAL},
{"GetDslPSD" 		,1,0,	RLCM_GET_DSL_PSD},
{"GetDslOthers"		,1,0,	RLCM_GET_DSL_ORHERS},
{"GetDslGi"		,1,0,	RLCM_GET_DSL_GI},
//end of tr069
{0,1,0,0}
};

const char err_mag[] = "ADSL ioctl failed!\n";
/*****************************************************************************/
//return 0 success, -1 no such command, -2 arg number error, -3 command fail

void argv_to_lower_case(char *str){
	int len,i;
	len=strlen(str);
	for(i=0;i<len;i++){
		if(str[i]>='A' && str[i]<='Z')
				str[i]-=('A'-'a');
	}
}
char *cmd_to_lower_case(char *str){
	int len,i;
	char *retstr;
	len=strlen(str);
	retstr=malloc(len*sizeof(char));
	for(i=0;i<len;i++){
		if(str[i]>='A' && str[i]<='Z')
				retstr[i]=str[i]-('A'-'a');
		else
				retstr[i]=str[i];
	}
	return retstr;
}

char *get_cmd_string(char *str){
	char *retstr=NULL;
	char *TableCmd;
	cmd_t *pCmd=cmdTable;
	int max_matching_len=0,len;
	//argv_to_lower_case(str); //change argv[1] to lower case
	while(pCmd->cmd!=NULL){
		TableCmd=cmd_to_lower_case(pCmd->cmd);
		if(strstr(TableCmd,str)==TableCmd){
			if(strlen(str)>max_matching_len){
				max_matching_len=strlen(str);
				retstr=pCmd->cmd;
			}
			else if(strlen(str)==max_matching_len){
				printf("Ambiguous command!\n");
				exit(0);
			}
		}
		free(TableCmd);
		pCmd++;
	}
	if(retstr==NULL){
		printf("No such command!\n");
		exit(0);
	}
	return retstr;
}

int AdslCMD(FILE *fp, int argc, char *argv[]) 
{
  cmd_t *pCmd=cmdTable;
  obcif_arg	myarg;
  char *cmdstr;
  
  cmdstr=get_cmd_string(argv[1]);//tylo, pCmd->cmd to lowercase string cmpstr
    while (pCmd->cmd!=NULL) {
	//if (strcmp(argv[1],pCmd->cmd)==0) {
	if(cmdstr==pCmd->cmd){	//tylo, cmdstr compare with pCmd->cmd
	    if (pCmd->argsize!=0) {  //use arg for API
	    	char *ptr;
	    	int *ptr32;
	    	int *ptr32_2;
		ptr32=ptr=malloc(pCmd->argsize);
	    	if (pCmd->setFlag==2) {  //need set arg from console
	    	    	if(pCmd->argsize==8){
				if (argc!=4) {
				printf("Error! Command need argument!\n");
				free(ptr);
	    	    		return -2;
	    	    		};
				ptr32_2=ptr32+4;
	    	    		*ptr32 = atoi(argv[2]); //small arguments, only WriteCfg can not work
	    	    		*ptr32_2=atoi(argv[3]);
	    	    		myarg.arg = (int)ptr32;
	    	    	}
	    	    	else{
	    	    		if (argc!=3) {
					printf("Error! Command need argument!\n");
					free(ptr);
	    	    			return -2;
	    	    		};
	    	    		*ptr32 = atoi(argv[2]); //small arguments, only WriteCfg can not work
	    	    		myarg.arg = (int)ptr32;
	    	    	}
		} else {  //get from DMT
	    	    myarg.arg = (int)ptr;
		};
	    	myarg.argsize = pCmd->argsize;
	        if (ioctl(fileno(fp), pCmd->funcID, &myarg) < 0) {
		    free(ptr);
	    	    return -3;
	    	};
		free(ptr);	    	
	    } else {  //no arg
	    	if (ioctl(fileno(fp), pCmd->funcID, NULL)<0)
	    	    return -3;
	    };
	    break;
	};
    	pCmd++;
    };
    if (pCmd->cmd!=NULL) {
        return 0;
    } else {
	printf("No such command!\n");
        return -1;
    };
}

int main(int argc, char *argv[])
{
	FILE		*fp;
	char		*procdevice, data[8];
	obcif_arg	myarg;
	int		arg;
	char rValueAdslDrv[64];

	procdevice = "/dev/adsl0";

	if (argc<2) {
	  printf("usage:adslctrl command args %d %s\n", argc, argv[1]);
	  exit(0);
	};

	argv_to_lower_case(argv[1]); //tylo, change argv[1] to lower case
	//printf("command args %d %s\n", argc, argv[1]);
	
	if ((fp = fopen(procdevice, "r")) == NULL) {
	    printf(stderr, "ERROR: failed to open %s, errno=%d\n",
		procdevice, errno);
	    exit(0);
	};
	memset(&myarg, 0, sizeof(obcif_arg));
	if (strcmp(argv[1],"start")==0) {
	    if (argc!=3) {
	        printf("start argument number error!\n");
	    } else {
	    	myarg.argsize = 4;
	    	arg = atoi(argv[2]);
	    	myarg.arg = (int)&arg;
	        if (ioctl(fileno(fp), RLCM_PHY_START_MODEM, &myarg) < 0) {
	    	    printf("ADSL start failed %x %x!\n", RLCM_PHY_START_MODEM, myarg.arg);
	    	};
	    };
	} else 	if (strcmp(argv[1],"connect")==0) {
	    myarg.argsize = 4;
	    arg = 1;
	    myarg.arg = (int)&arg;
	    if (ioctl(fileno(fp), RLCM_PHY_ENABLE_MODEM, &myarg) < 0) {
	    	printf("ADSL connect failed %x!\n", RLCM_PHY_ENABLE_MODEM);
	    };
	} else if (strcmp(argv[1],"disconnect")==0) {
	    if (ioctl(fileno(fp), RLCM_PHY_DISABLE_MODEM, NULL) < 0) {
	    	printf("ADSL disconnect failed! %x\n", RLCM_PHY_DISABLE_MODEM);
	    };
	} else if (strcmp(argv[1],"getdriverversion")==0) {
		char rValueAdslDrv[256];
		myarg.argsize = 256;
		myarg.arg = (int)&rValueAdslDrv[0];

		if (ioctl(fileno(fp), RLCM_GET_DRIVER_VERSION, &myarg)<0) {
			strcpy(rValueAdslDrv, "Error");
		}
		printf("getDriverVersion=%s\n", rValueAdslDrv);
	}
	else if (strcmp(argv[1],"getadslmode")==0) {
		char mode=0;
		myarg.argsize = sizeof(char);
		myarg.arg = (int)&mode;
		char rValueAdslDrv[16];

		if (ioctl(fileno(fp), RLCM_GET_SHOWTIME_XDSL_MODE, &myarg) < 0) {
			mode=0;
		}

		if (mode == 1) { // ADSL1
			int vInt;
			myarg.argsize = sizeof(int);
			myarg.arg = (int)&vInt;
			if (ioctl(fileno(fp), RLCM_GET_ADSL_MODE, &myarg) < 0) {
				printf("Get ADSL Mode failed %x!\n", RLCM_GET_SHOWTIME_XDSL_MODE);
				goto stop_adslctrl;
			}

			if (vInt==1)
				strcpy(rValueAdslDrv, "T1.413");
			if (vInt==2)
				strcpy(rValueAdslDrv, "G.Dmt");
		}
		else if (mode == 2)  // G.Lite
			strcpy(rValueAdslDrv, "G.Lite");
		else if (mode == 3)  // ADSL2
			strcpy(rValueAdslDrv, "ADSL2");
		else if (mode == 5)  // ADSL2+
			strcpy(rValueAdslDrv, "ADSL2+");
		else {
			strcpy(rValueAdslDrv, "NONE");
		}

		printf("getADSLMode=%s\n", rValueAdslDrv);
	} else if (strcmp(argv[1],"getupstream")==0) {
		Modem_LinkSpeed Link_Speed;
		Link_Speed.downstreamRate = 0;
		myarg.argsize = sizeof(Link_Speed);
		myarg.arg = (int)&Link_Speed;
		if (ioctl(fileno(fp), RLCM_GET_LINK_SPEED, &myarg) < 0) {
			Link_Speed.upstreamRate = 0;
		}

		printf("getUpStream=%d\n", Link_Speed.upstreamRate);
	} else if (strcmp(argv[1],"getdownstream")==0) {
		Modem_LinkSpeed Link_Speed;
		Link_Speed.downstreamRate = 0;
		myarg.argsize = sizeof(Link_Speed);
		myarg.arg = (int)&Link_Speed;
		if (ioctl(fileno(fp), RLCM_GET_LINK_SPEED, &myarg) < 0) {
			Link_Speed.downstreamRate = 0;
		}

		printf("getDownStream=%d\n", Link_Speed.downstreamRate);

		//Modem_LinkSpeed Link_Rate;
		//Link_Rate.downstreamRate = 0;
		//myarg.argsize = sizeof(Link_Rate);
		//myarg.arg = (int)&Link_Rate;
		//if (ioctl(fileno(fp), RLCM_GET_LINE_RATE, &myarg) < 0)
		//{
		//	printf("List API command failed! %x\n", RLCM_GET_LINE_RATE);
		//	goto stop_adslctrl;
		//}

		//Modem_LinkSpeed ATT_Rate;
		//ATT_Rate.downstreamRate = 0;
		//myarg.argsize = sizeof(ATT_Rate);
		//myarg.arg = (int)&ATT_Rate;
		//if (ioctl(fileno(fp), RLCM_GET_GET_ATT_RATE, &myarg) < 0)
		//{
		//	printf("List API command failed! %x\n", RLCM_GET_GET_ATT_RATE);
		//	goto stop_adslctrl;
		//}

		//printf(",downLinkrate=%d,upLinkrate=%d,downATTrate=%d,upATTrate=%d\n"
		//	, Link_Rate.downstreamRate, Link_Rate.upstreamRate
		//	, ATT_Rate.downstreamRate, ATT_Rate.upstreamRate
		//	);
	} else if (strcmp(argv[1],"?")==0) {
	    if (ioctl(fileno(fp), RLCM_CMD_API, NULL) < 0) {
	    	printf("List API command failed! %x\n", RLCM_CMD_API);
	    };
	} else if (strcmp(argv[1],"test")==0) {  
	    //for v1.0.278, auto reconnect if rate is OK, disconnect if rate is below input arg
	    int rate_drop=0;
	    int mode, last_rate=0;
	    Modem_LinkSpeed Link_Speed;
	    
	    if (argc<3) {
	    	printf("usage: adslctrl test mode\n");
	        goto stop_adslctrl;
	    };
	    mode = atoi(argv[2]);
	    if ((mode<1)||(mode>3)) {
	    	printf("adsl link mode error, only 1, 2, 3 is allowed\n");
	        goto stop_adslctrl;
	    };
	    
	    while (rate_drop==0) {
retry_test:	    	
	    	//disconnect
	        if (ioctl(fileno(fp), RLCM_PHY_DISABLE_MODEM, NULL) < 0) {
	    	    printf("ADSL disconnect failed! %x\n", RLCM_PHY_DISABLE_MODEM);
	        };
	        //wait 1 sec
	        sleep(3);
	        //reconnect
	        myarg.argsize = 4;
	        myarg.arg = (int)&mode;
	        if (ioctl(fileno(fp), RLCM_PHY_ENABLE_MODEM, &myarg) < 0) {
	    	    printf("ADSL connect failed %x!\n", RLCM_PHY_ENABLE_MODEM);
	    	    goto retry_test;
	        };
	        //wait show time
wait_showtime:
	        sleep(1);
	        Link_Speed.downstreamRate = 0;
	        myarg.argsize = sizeof(Link_Speed);
	        myarg.arg = (int)&Link_Speed;
	        if (ioctl(fileno(fp), RLCM_GET_LINK_SPEED, &myarg) < 0) {
	    	    printf("List API command failed! %x\n", RLCM_GET_LINK_SPEED);
	        };
	        if (Link_Speed.downstreamRate==0)
	            goto wait_showtime;
	        //show time
	        sleep(10);
	        if (last_rate!=0) {
	            int diff;
	            diff = Link_Speed.downstreamRate-last_rate;
	            if ( (diff<0)&&(abs(diff)>(last_rate/2)) ) {
	                printf("Stop adsl test! Link rate too low. Last: %d\n", last_rate);
	                rate_drop=1;
	                goto stop_adslctrl;
	            } 
	        };
	        last_rate = Link_Speed.downstreamRate;
	    };
	} else if (strcmp(argv[1],"ioctl")==0) {
	    if (argc!=3) {
	        printf("ioctl argument number error!\n");
	    } else {
	        unsigned int id=atoi(argv[2]);
	        unsigned char ifr[256];

	        if (id==-1) {
	            printf("No such ioctl command!\n");
	        } else {
	    	    myarg.argsize = 256;
	    	    myarg.arg = (int)ifr;
	    	    memset(ifr, 0, 256);
	    	    printf("ADSL ioctl %x\n", id);
	            if (ioctl(fileno(fp), id, &myarg) < 0) {
	    	        printf("ADSL ioctl failed! %x\n", id);
	            } else {
	            	int i;
	                for (i=0;i<32;i++) {
	                  printf("%02X: %02X %02X %02X %02X %02X %02X %02X %02X\n", i*8,
	                      ifr[i*8+0], ifr[i*8+1], ifr[i*8+2], ifr[i*8+3], 
	                      ifr[i*8+4], ifr[i*8+5], ifr[i*8+6], ifr[i*8+7]);
	                };
	            };
	        };
	    };
	} else if (strcmp(argv[1],"adslver")==0) {
		myarg.argsize = RLCM_DRIVER_VERSION_SIZE;
		myarg.arg = (int)&rValueAdslDrv[0];
		if (ioctl(fileno(fp), RLCM_GET_DRIVER_VERSION, &myarg) < 0) {
		    printf(err_mag);
		};
		printf("Version: %s\n", rValueAdslDrv);
	} else if (strcmp(argv[1],"adslbuild")==0) {
		myarg.argsize = RLCM_DRIVER_BUILD_SIZE;
		myarg.arg = (int)&rValueAdslDrv[0];
		if (ioctl(fileno(fp), RLCM_GET_DRIVER_BUILD, &myarg) < 0) {
		    printf(err_mag);
		};
		printf("Build: %s\n", rValueAdslDrv);
	} else {
	    //Search cmd table
	    AdslCMD(fp, argc, argv);
	};

stop_adslctrl:
	fclose(fp);
	exit(0);
}

/*****************************************************************************/
