/*
 *      Web server handler routines for NET
 *      Authors: 
 *
 */

/*-- System inlcude files --*/
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <sys/wait.h>
#include <sys/socket.h>
#include <semaphore.h>
#ifdef EMBED
#include <linux/config.h>
#else
#include "../../../uClibc/include/linux/autoconf.h"
#endif

/* for ioctl */
#include <sys/ioctl.h>
#include <net/if.h>

#include "../webs.h"
#include "fmdefs.h"
#include "mib.h"
#include "utility.h"
#include "debug.h"
#include "../defs.h"
#include "adsl_drv.h"


#define UBR_WITHOUT_PCR   0
#define UBR_WITH_PCR   1
#define CBR            2
#define NO_RT_VBR    3
#define RT_VBR      4

static int mib2web(MIB_CE_ATM_VC_Tp mibentry,struct atmvc_entryx* webentry)
{
	webentry->svtype = mibentry->svtype;
	webentry->vpi = mibentry->vpi;
	webentry->vci = mibentry->vci;
	webentry->pcr = mibentry->pcr;
	webentry->scr = mibentry->scr;
	webentry->mbs = mibentry->mbs;
	webentry->encap = (mibentry->encap==1?0:1);
	webentry->napt = mibentry->napt;

	if(mibentry->cmode==ADSL_PPPoA||mibentry->cmode==ADSL_RT1483)
		return -1;
	webentry->cmode = mibentry->cmode;
	//webentry->brmode = mibentry->brmode;
	strcpy(webentry->pppUsername,mibentry->pppUsername);
	strcpy(webentry->pppPassword, mibentry->pppPassword);
	webentry->pppAuth = mibentry->pppAuth;
#ifdef _CWMP_MIB_ /*jiunming, for cwmp-tr069*/
	strcpy(webentry->pppServiceName, mibentry->pppServiceName);
#endif
	webentry->pppCtype = mibentry->pppCtype;
	webentry->ipDhcp = mibentry->ipDhcp;
	*(unsigned long*)&(webentry->ipAddr) = ntohl(*(unsigned long*)&(mibentry->ipAddr));
	*(unsigned long*)&(webentry->remoteIpAddr) = ntohl(*(unsigned long*)&(mibentry->remoteIpAddr));
	*(unsigned long*)&(webentry->netMask) = ntohl(*(unsigned long*)&(mibentry->netMask));
#ifdef CONFIG_EXT_SWITCH
	webentry->vlan = mibentry->vlan;
	webentry->vid = mibentry->vid;
	webentry->vprio = mibentry->vprio;
	webentry->vpass = mibentry->vpass;
#endif
	webentry->qos = mibentry->qosenable;
#ifdef _PRMT_X_CT_COM_PPPOEv2_
	webentry->PPPoEProxyEnable = mibentry->PPPoEProxyEnable;
	webentry->PPPoEProxyMaxUser = mibentry->PPPoEProxyMaxUser;
#endif
#ifdef CTC_WAN_NAME
	webentry->applicationtype = mibentry->applicationtype;
#endif
	webentry->dgw = mibentry->dgw;
	webentry->ifIndex = mibentry->ifIndex;
#ifdef PPPOE_PASSTHROUGH
	if(mibentry->cmode==ADSL_PPPoE){
		if(mibentry->brmode==BRIDGE_DISABLE)
			webentry->brmode=0;
		else if(mibentry->brmode==BRIDGE_PPPOE)
			webentry->brmode=1;
		else
			webentry->brmode=0;
	}else
		webentry->brmode=0;
#endif
	if(mibentry->itfGroup==0)
		webentry->itfGroup=0;
	else{
		unsigned char vChar,vcgroup;
		int i;
		unsigned short group=0;
#ifdef CONFIG_EXT_SWITCH
		MIB_CE_SW_PORT_T Port;

		vcgroup = mibentry->itfGroup;

	printf("\nitfgroup=%d\n",vcgroup);
		for(i=0;i<4;i++){
			if( mib_chain_get(MIB_SW_PORT_TBL, i, (void *)&Port) ){
				if(vcgroup == Port.itfGroup)
					group|=(1<<i);
			}
		}
#endif

#ifdef WLAN_SUPPORT
		mib_get(MIB_WLAN_ITF_GROUP, (void *)&vChar);
		if(vcgroup == vChar)
			group|=0x10;
#ifdef WLAN_MBSSID
		mib_get(MIB_WLAN_VAP0_ITF_GROUP, (void *)&vChar);
		if(vcgroup == vChar)
			group|=0x20;
		mib_get(MIB_WLAN_VAP1_ITF_GROUP, (void *)&vChar);
		if(vcgroup == vChar)
			group|=0x40;
		mib_get(MIB_WLAN_VAP2_ITF_GROUP, (void *)&vChar);
		if(vcgroup == vChar)
			group|=0x80;
		mib_get(MIB_WLAN_VAP3_ITF_GROUP, (void *)&vChar);
		if(vcgroup == vChar)
			group|=0x100;
#endif //WLAN_MBSSID		
#endif //WLAN_SUPPORT
		webentry->itfGroup=group;

	}
	return 0;

}

static int web2mib(struct atmvc_entryx* webentry,MIB_CE_ATM_VC_Tp mibentry)
{
	switch(webentry->svtype){
		case UBR_WITHOUT_PCR:
		case UBR_WITH_PCR:
			mibentry->qos=ATMQOS_UBR;
			break;
		case CBR:
			mibentry->qos=ATMQOS_CBR;
			break;
		case NO_RT_VBR:
			mibentry->qos=ATMQOS_VBR_NRT;
			break;
		case RT_VBR:
			mibentry->qos=ATMQOS_VBR_RT;
			break;
		default:
			mibentry->qos=ATMQOS_UBR;
	}
	mibentry->svtype = webentry->svtype;
	mibentry->vpi = webentry->vpi;
	mibentry->vci = webentry->vci;
	mibentry->pcr = webentry->pcr;
	mibentry->scr = webentry->scr;
	mibentry->mbs = webentry->mbs;
	mibentry->encap = (webentry->encap==1?0:1);
	mibentry->napt = webentry->napt;
	mibentry->cmode = webentry->cmode;
	//mibentry->brmode = webentry->brmode;
	strcpy(mibentry->pppUsername,webentry->pppUsername);
	strcpy(mibentry->pppPassword, webentry->pppPassword);
	mibentry->pppAuth = webentry->pppAuth;
#ifdef _CWMP_MIB_ /*jiunming, for cwmp-tr069*/
	strcpy(mibentry->pppServiceName, webentry->pppServiceName);
#endif
	mibentry->pppCtype = webentry->pppCtype;
	mibentry->ipDhcp = webentry->ipDhcp;
	*(unsigned long*)&(mibentry->ipAddr) = htonl(*(unsigned long*)&(webentry->ipAddr));
	*(unsigned long*)&(mibentry->remoteIpAddr) = htonl(*(unsigned long*)&(webentry->remoteIpAddr));
	*(unsigned long*)&(mibentry->netMask) = htonl(*(unsigned long*)&(webentry->netMask));
#ifdef CONFIG_EXT_SWITCH
	mibentry->vlan = webentry->vlan;
	mibentry->vid = webentry->vid;
	mibentry->vprio = webentry->vprio;
	mibentry->vpass = webentry->vpass;
#endif
	mibentry->qosenable = webentry->qos;
#ifdef _PRMT_X_CT_COM_PPPOEv2_
	mibentry->PPPoEProxyEnable = webentry->PPPoEProxyEnable;
	mibentry->PPPoEProxyMaxUser = webentry->PPPoEProxyMaxUser;
#endif
#ifdef CTC_WAN_NAME
	mibentry->applicationtype = webentry->applicationtype;
#endif
	mibentry->dgw = webentry->dgw;
#ifdef PPPOE_PASSTHROUGH
	if (mibentry->cmode == ADSL_BR1483){
		if(mibentry->brmode!=BRIDGE_PPPOE)
			mibentry->brmode = BRIDGE_ETHERNET;}
	else if(mibentry->cmode == ADSL_MER1483)
		mibentry->brmode = BRIDGE_DISABLE;
	else{
		if(webentry->brmode==0)
			mibentry->brmode=BRIDGE_DISABLE;
		else
			mibentry->brmode=BRIDGE_PPPOE;
	}
#endif

#ifdef _PRMT_X_CT_COM_WANEXT_
#ifdef CTC_WAN_NAME
	switch(mibentry->applicationtype){
		case 0:
			mibentry->ServiceList = X_CT_SRV_INTERNET |X_CT_SRV_TR069;
			break;
		case 1: 
			mibentry->ServiceList = X_CT_SRV_INTERNET;
			break;
		case 2: 
			mibentry->ServiceList = X_CT_SRV_TR069;
			break;
		case 3: 
			mibentry->ServiceList = X_CT_SRV_OTHER;
			break;
		default:
			mibentry->ServiceList = X_CT_SRV_INTERNET;
			break;
	}
#endif
#endif 
#ifdef CTC_WAN_NAME
	if(mibentry->applicationtype==2){
		mibentry->napt=0;
		mibentry->dgw=0;
	}
#endif

	return 0;

}

/*****************************
** wan if list
*/
int listWanif(int eid, webs_t wp, int argc, char_t ** argv)
{
#define _LK_FLAG_RT		0x01
#define _LK_FLAG_BR		0x02
	struct atmvc_entryx	entry;
	char*			type = NULL;
	int				cnt = 1;
	int				index = 0;
	int				flag = 0;
	int				lineno = __LINE__;
	unsigned char	ipAddr[16];		//IPַ
	unsigned char	remoteIpAddr[16];	//ȱʡ
	unsigned char	netMask[16];	//

	MIB_CE_ATM_VC_T mibentry;
	int mibtotal,i;

	char wanname[MAX_NAME_LEN];

	_TRACE_CALL;

   	if (ejArgs(argc, argv, T("%s"), &type) < 1) 
	{
		flag = _LK_FLAG_RT | _LK_FLAG_BR;
   	}
	else if(strcmp(type, "rt") == 0)
	{
		flag = _LK_FLAG_RT;
	}
	else if(strcmp(type, "br") == 0)
	{
		flag = _LK_FLAG_BR;
	}
	else if(strcmp(type, "all") == 0)
	{
		flag = _LK_FLAG_BR;
	}
	else return -1;

	/************Place your code here, do what you want to do! ************/
	/************Place your code here, do what you want to do! ************/
	
	memset(&entry,0,sizeof(entry));

	mibtotal = mib_chain_total(MIB_ATM_VC_TBL);
	for(i=0;i<mibtotal;i++){
		if(mib_chain_get(MIB_ATM_VC_TBL,i,&mibentry)!=1)
			continue;
		/************Place your code here, do what you want to do! ************/
		/************Place your code here, do what you want to do! ************/
		getWanName(&mibentry, wanname);
		int tmp=mib2web(&mibentry,&entry);
		if(tmp==-1)
			continue;

		if(entry.cmode == 0 && !(flag & _LK_FLAG_BR)) continue;
		else if(entry.cmode != 0 && !(flag & _LK_FLAG_RT)) continue;

		websWrite(wp, T("push(new it_nr(\"%s\"" _PTI _PTI _PTI _PTI _PTI _PTI \
			_PTI _PTI _PTI _PTI _PTS _PTI _PTS \
			_PTI _PTI _PTS _PTS _PTS _PTI _PTI _PTI _PTI _PTI _PTI _PTI _PTI _PTI "));\n"), 
			wanname, _PME(vpi), _PME(qos), _PME(vci), _PME(pcr), _PME(scr), _PME(mbs), 
			_PME(encap), _PME(napt), _PME(cmode), _PME(brmode), 
			_PME(pppUsername)/*, _PME(pppPassword)*/, _PME(pppAuth), _PME(pppServiceName),
			_PME(pppCtype), _PME(ipDhcp),
			_PMEIP(ipAddr), _PMEIP(remoteIpAddr), _PMEIP(netMask),
			_PME(vlan), _PME(vid), _PME(vprio), _PME(vpass), _PME(itfGroup),
			_PME(PPPoEProxyEnable), _PME(PPPoEProxyMaxUser), _PME(applicationtype), _PME(ifIndex)
			);

	}

check_err:
	_TRACE_LEAVEL;
	return 0;
}


/*****************************
** Internet
*/
int initPageAdsl(int eid, webs_t wp, int argc, char_t ** argv)
{
	struct atmvc_entryx	entry;
	int				pppnumleft = 5;
	int				cnt = 0;
	int				index = 0;
	unsigned char	ipAddr[16];		//IPַ
	unsigned char	remoteIpAddr[16];	//ȱʡ
	unsigned char	netMask[16];	//
	unsigned long	fstdns = 0;	//ȱʡDNS
	unsigned long	secdns = 0;	//ѡDNS
	int				lineno = __LINE__;

	_TRACE_CALL;

	/************Place your code here, do what you want to do! ************/
	/*test code*
	cnt = 1;
	pppnumleft = 7;

	memset(&entry, 0, sizeof(entry));
	entry.svtype = 0;
	entry.vpi = 0;
	entry.vci = 32;
	entry.pcr = 0;
	entry.scr = 0;
	entry.mbs = 0;
	entry.encap = 0;
	entry.napt = 0;
	entry.cmode = 2;
	entry.brmode = 1;
	strcpy(entry.pppUsername, "test");
	strcpy(entry.pppPassword, "test");
	entry.pppAuth = 0;
	strcpy(entry.pppServiceName, "test");
	entry.pppCtype = 0;
	entry.ipDhcp = 0;
	*(unsigned long*)&(entry.ipAddr) = 0x77777777;
	*(unsigned long*)&(entry.remoteIpAddr) = 0x77777777;
	*(unsigned long*)&(entry.netMask) = htonl(0xFFFFFF00);
	entry.vlan = 1;
	entry.vid = 0;
	entry.vprio = 0;
	entry.vpass = 0;
	entry.itfGroup = 0x48;
	entry.qos = 1;
	entry.PPPoEProxyEnable = 1;
	entry.PPPoEProxyMaxUser = 0;
	entry.applicationtype = 2;
	************Place your code here, do what you want to do! ************/
	MIB_CE_ATM_VC_T mibentry;
	int mibtotal,i;
	char wanname[MAX_NAME_LEN];

	memset(&entry,0,sizeof(entry));

		_PUT_INT(pppnumleft);
	mibtotal = mib_chain_total(MIB_ATM_VC_TBL);
	for(i=0;i<mibtotal;i++){
		if(mib_chain_get(MIB_ATM_VC_TBL,i,&mibentry)!=1)
			continue;
	
	//	_PUT_INT(pppnumleft);
	//	_PUT_IP(fstdns);
	//	_PUT_IP(secdns);

//	for(index = 0; index < cnt; index++)
		{
			/************Place your code here, do what you want to do! ************/
			/************Place your code here, do what you want to do! ************/

			getWanName(&mibentry, wanname);
			int tmp=mib2web(&mibentry,&entry);
			if(tmp==-1)
				continue;

			websWrite(wp, T("push(new it_nr(\"%s\"" _PTI _PTI _PTI _PTI _PTI _PTI \
				_PTI _PTI _PTI _PTI \
				_PTS _PTS _PTI _PTS \
				_PTI _PTI \
				_PTS _PTS _PTS _PTI\
				_PTI _PTI _PTI _PTI _PTI \
				_PTI _PTI _PTI _PTI "));\n"), 
				wanname, _PME(svtype), _PME(vpi), _PME(vci), _PME(pcr), _PME(scr), _PME(mbs), 
				_PME(encap), _PME(napt), _PME(cmode), _PME(brmode), 
				_PME(pppUsername), _PME(pppPassword), _PME(pppAuth), _PME(pppServiceName),
				_PME(pppCtype), _PME(ipDhcp),
				_PMEIP(ipAddr), _PMEIP(remoteIpAddr), _PMEIP(netMask), _PME(dgw),
				_PME(vlan), _PME(vid), _PME(vprio), _PME(vpass), _PME(itfGroup),
				_PME(qos), _PME(PPPoEProxyEnable), _PME(PPPoEProxyMaxUser), _PME(applicationtype)
				);
		}
	}

check_err:
	_TRACE_LEAVEL;
	return 0;
}

extern void resolveServiceDependency(idx);
extern unsigned char if_find_index(int cmode, unsigned int map);
#ifdef CONFIG_EXT_SWITCH
static int setgroupbyweb(int itfgroup, char *action, MIB_CE_ATM_VC_Tp pEntry)
{
	int fsw0=0, fsw1=0, fsw2=0, fsw3=0, fwan=0;
	unsigned char gsw0=0, gsw1=0, gsw2=0, gsw3=0;
#ifdef WLAN_SUPPORT
	int fwlan0=0, fwlan1=0, fwlan2=0, fwlan3=0, fwlan4=0;
	unsigned char gwlan0=0, gwlan1=0, gwlan2=0, gwlan3=0, gwlan4=0;
#endif //WLAN_SUPPORT
	MIB_CE_SW_PORT_T Port;
	unsigned char group=0,def_group=0;

	int i;

#ifdef WLAN_SUPPORT
#ifndef WLAN_MBSSID
	itfgroup&=0x1f;
#endif
#else
	itfgroup&=0xf;
#endif

	if(!strcmp(action,"add")){
		if(itfgroup==0)
	//		return 0;
	             goto check_mib;
		fwan=1;
	}else if(!strcmp(action,"modify")){
		if(itfgroup==0)
		{
			MIB_CE_ATM_VC_T pvcEntry;
			unsigned int vcNum;	
			if( pEntry->itfGroup==def_group )
				//return 0; //no change
				 goto check_mib;
			//check if other wan interfaces are in the same group
			vcNum = mib_chain_total(MIB_ATM_VC_TBL);
			for (i=0; i<vcNum; i++)
			{
				if (!mib_chain_get(MIB_ATM_VC_TBL, i, (void *)&pvcEntry))
					continue;
				if( (pvcEntry.ifIndex!=pEntry->ifIndex) &&
				    (pvcEntry.itfGroup==pEntry->itfGroup) )
				{
					//just remove the pEntry from this group
					pEntry->itfGroup=def_group;
				//	return 0;
				       goto check_mib;
				}
			}
			fwan=0; //delete
		}else
			fwan=1;
	}

       if(pEntry->cmode==ADSL_PPPoE) {
        
		MIB_CE_ATM_VC_T pvcEntry;
		unsigned int vcNum;
                 vcNum = mib_chain_total(MIB_ATM_VC_TBL);
			for (i=0; i<vcNum; i++)
			{
				if (!mib_chain_get(MIB_ATM_VC_TBL, i, (void *)&pvcEntry))
					continue;
				if( (pvcEntry.cmode==ADSL_PPPoE)&&(pvcEntry.ifIndex!=pEntry->ifIndex) &&
				    (pvcEntry.vpi==pEntry->vpi) &&(pvcEntry.vci==pEntry->vci))
				{   
				       printf("************same pvc have multi pppoe*********** \n");
					//just remove the pEntry from this group
					pEntry->itfGroup=pvcEntry.itfGroup;
			
				}
			}

	   } 
	printf("\nitfgroup=%d,fwan=%d,pEntry->group=%d\n",itfgroup,fwan,pEntry->itfGroup);

	if((itfgroup&0x1)!=0)	fsw0=1;
	if((itfgroup&0x2)!=0)	fsw1=1;
	if((itfgroup&0x4)!=0)	fsw2=1;
	if((itfgroup&0x8)!=0)	fsw3=1;
#ifdef WLAN_SUPPORT
	if((itfgroup&0x10)!=0)	fwlan0=1;
#ifdef WLAN_MBSSID
	if((itfgroup&0x20)!=0)	fwlan1=1;
	if((itfgroup&0x40)!=0)	fwlan2=1;
	if((itfgroup&0x80)!=0)	fwlan3=1;
	if((itfgroup&0x100)!=0)	fwlan4=1;
#endif
#endif
	printf("\n%d %d %d %d\n",fsw0,fsw1,fsw2,fsw3);
	{
#ifdef WLAN_SUPPORT
		mib_get(MIB_WLAN_ITF_GROUP, (void *)&gwlan0);
#ifdef WLAN_MBSSID
		mib_get(MIB_WLAN_VAP0_ITF_GROUP, (void *)&gwlan1);
		mib_get(MIB_WLAN_VAP1_ITF_GROUP, (void *)&gwlan2);
		mib_get(MIB_WLAN_VAP2_ITF_GROUP, (void *)&gwlan3);
		mib_get(MIB_WLAN_VAP3_ITF_GROUP, (void *)&gwlan4);
#endif //WLAN_MBSSID		
#endif //WLAN_SUPPORT
		if( mib_chain_get(MIB_SW_PORT_TBL, 0, (void *)&Port) )
			gsw0=Port.itfGroup;
		if( mib_chain_get(MIB_SW_PORT_TBL, 1, (void *)&Port) )
			gsw1=Port.itfGroup;
		if( mib_chain_get(MIB_SW_PORT_TBL, 2, (void *)&Port) )
			gsw2=Port.itfGroup;
		if( mib_chain_get(MIB_SW_PORT_TBL, 3, (void *)&Port) )
			gsw3=Port.itfGroup;
	}

	{
		if( (fwan==1)&&(pEntry->itfGroup!=0) )  group=pEntry->itfGroup;
#ifdef WLAN_SUPPORT
		else if( (fwlan0==1)&&(gwlan0!=0) ) group=gwlan0;
#ifdef WLAN_MBSSID
		else if( (fwlan1==1)&&(gwlan1!=0) ) group=gwlan1;
		else if( (fwlan2==1)&&(gwlan2!=0) ) group=gwlan2;
		else if( (fwlan3==1)&&(gwlan3!=0) ) group=gwlan3;
		else if( (fwlan4==1)&&(gwlan4!=0) ) group=gwlan4;
#endif //WLAN_MBSSID		
#endif //WLAN_SUPPORT
		else if( (fsw0==1)&&(gsw0!=0) ) group=gsw0;
		else if( (fsw1==1)&&(gsw1!=0) ) group=gsw1;
		else if( (fsw2==1)&&(gsw2!=0) ) group=gsw2;
		else if( (fsw3==1)&&(gsw3!=0) ) group=gsw3;
		else group=0;

		printf("\n%d %d %d %d\n",gsw0,gsw1,gsw2,gsw3);
		if( (group!=0) && 
		      (((fwan==1)&&(pEntry->itfGroup!=0)&&(group!=pEntry->itfGroup)) ||
#ifdef WLAN_SUPPORT
		       ((fwlan0==1)&&(gwlan0!=0)&&(group!=gwlan0)) ||
#ifdef WLAN_MBSSID
		       ((fwlan1==1)&&(gwlan1!=0)&&(group!=gwlan1)) ||
		       ((fwlan2==1)&&(gwlan2!=0)&&(group!=gwlan2)) ||
		       ((fwlan3==1)&&(gwlan3!=0)&&(group!=gwlan3)) ||
		       ((fwlan4==1)&&(gwlan4!=0)&&(group!=gwlan4)) ||
#endif //WLAN_MBSSID		
#endif //WLAN_SUPPORT
		       ((fsw0==1)&&(gsw0!=0)&&(group!=gsw0)) ||
		       ((fsw1==1)&&(gsw1!=0)&&(group!=gsw1)) ||
		       ((fsw2==1)&&(gsw2!=0)&&(group!=gsw2)) ||
		       ((fsw3==1)&&(gsw3!=0)&&(group!=gsw3))
		  ))
		{
			//remove single lan port group
			unsigned char groupmap=0;
			MIB_CE_ATM_VC_T pvcEntry;
			unsigned int vcNum;	

			vcNum = mib_chain_total(MIB_ATM_VC_TBL);
			for (i=0; i<vcNum; i++)
			{
				if (!mib_chain_get(MIB_ATM_VC_TBL, i, (void *)&pvcEntry))
					continue;
				if(pvcEntry.ifIndex!=pEntry->ifIndex)
					groupmap|=(1<<(pvcEntry.itfGroup));
			}
			printf("\ngroupmap=%x\n",groupmap);
#ifdef WLAN_SUPPORT
			if((fwlan0==0)&&(gwlan0!=0)&&((groupmap&(1<<gwlan0))==0))	mib_set(MIB_WLAN_ITF_GROUP, (void *)&def_group);
#ifdef WLAN_MBSSID
			if((fwlan1==0)&&(gwlan1!=0)&&((groupmap&(1<<gwlan1))==0))	mib_set(MIB_WLAN_VAP0_ITF_GROUP, (void *)&def_group);
			if((fwlan2==0)&&(gwlan2!=0)&&((groupmap&(1<<gwlan2))==0))	mib_set(MIB_WLAN_VAP1_ITF_GROUP, (void *)&def_group);
			if((fwlan3==0)&&(gwlan3!=0)&&((groupmap&(1<<gwlan3))==0))	mib_set(MIB_WLAN_VAP2_ITF_GROUP, (void *)&def_group);
			if((fwlan4==0)&&(gwlan4!=0)&&((groupmap&(1<<gwlan4))==0))	mib_set(MIB_WLAN_VAP3_ITF_GROUP, (void *)&def_group);
#endif
#endif
			if((fsw0==0)&&(gsw0!=0)&&((groupmap&(1<<gsw0))==0)){
				if( mib_chain_get(MIB_SW_PORT_TBL, 0, (void *)&Port) ){
					Port.itfGroup=def_group;
					mib_chain_update( MIB_SW_PORT_TBL, (unsigned char*)&Port, 0);
				}
			}
			if((fsw1==0)&&(gsw1!=0)&&((groupmap&(1<<gsw1))==0)){
				if( mib_chain_get(MIB_SW_PORT_TBL, 1, (void *)&Port) ){
					Port.itfGroup=def_group;
					mib_chain_update( MIB_SW_PORT_TBL, (unsigned char*)&Port, 1);
				}
			}
			if((fsw2==0)&&(gsw2!=0)&&((groupmap&(1<<gsw2))==0)){
				if( mib_chain_get(MIB_SW_PORT_TBL, 2, (void *)&Port) ){
					Port.itfGroup=def_group;
					mib_chain_update( MIB_SW_PORT_TBL, (unsigned char*)&Port, 2);
				}
			}
			if((fsw3==0)&&(gsw3!=0)&&((groupmap&(1<<gsw3))==0)){
				if( mib_chain_get(MIB_SW_PORT_TBL, 3, (void *)&Port) ){
					Port.itfGroup=def_group;
					mib_chain_update( MIB_SW_PORT_TBL, (unsigned char*)&Port, 3);
				}
			}
	
			group=0;  // overwrite mix group
		}	
	}

	if( (fwan==1)&&(group==0) ) //found a avaliable group num;
	{
		struct itfInfo itfs[16];
		int ifnum,i;
		for( i=1;i<IFGROUP_NUM;i++ ) //from 1 to IFGROUP_NUM-1, 0:default group
		{
			ifnum = get_group_ifinfo(itfs, 16, i);
			if(ifnum==0)
			{
				group = i;
				break;
			}
		}
		
		if(group==0) return -1;
	}else if( fwan==0 ) //delete
		group = pEntry->itfGroup;

	//printf("\ngroup=%d\n",group);

	//enable the portmapping function
	if(fwan==1) //add
	{
		unsigned char mode=0;
		mib_get(MIB_MPMODE, (void *)&mode);
		mode |= 0x01;
		mib_set(MIB_MPMODE, (void *)&mode);
	}
	
	//update the wan's group
	if( fwan==1 )
		pEntry->itfGroup = group; //call mib_chain_update() after reutrn 
	else
		pEntry->itfGroup = def_group; //call mib_chain_update() after reutrn 

#ifdef WLAN_SUPPORT
	{ //update wlan's group
		if( fwlan0==1 ) //add
			mib_set(MIB_WLAN_ITF_GROUP, (void *)&group);
		else if( (fwlan0==0) && ( gwlan0==group ) ) //delete
			mib_set(MIB_WLAN_ITF_GROUP, (void *)&def_group);
#ifdef WLAN_MBSSID
		if( fwlan1==1 ) //add
			mib_set(MIB_WLAN_VAP0_ITF_GROUP, (void *)&group);
		else if( (fwlan1==0) && ( gwlan1==group ) ) //delete
			mib_set(MIB_WLAN_VAP0_ITF_GROUP, (void *)&def_group);
		if( fwlan2==1 ) //add
			mib_set(MIB_WLAN_VAP1_ITF_GROUP, (void *)&group);
		else if( (fwlan2==0) && ( gwlan2==group ) ) //delete
			mib_set(MIB_WLAN_VAP1_ITF_GROUP, (void *)&def_group);
		if( fwlan3==1 ) //add
			mib_set(MIB_WLAN_VAP2_ITF_GROUP, (void *)&group);
		else if( (fwlan3==0) && ( gwlan3==group ) ) //delete
			mib_set(MIB_WLAN_VAP2_ITF_GROUP, (void *)&def_group);
		if( fwlan4==1 ) //add
			mib_set(MIB_WLAN_VAP3_ITF_GROUP, (void *)&group);
		else if( (fwlan4==0) && ( gwlan4==group ) ) //delete
			mib_set(MIB_WLAN_VAP3_ITF_GROUP, (void *)&def_group);
#endif //WLAN_MBSSID		
	}
#endif //WLAN_SUPPORT

	{//update lan port's group
		int i;
		for( i=0;i<SW_PORT_NUM;i++ )
		{
			if( !mib_chain_get(MIB_SW_PORT_TBL, i, (void *)&Port) )
				continue;
			if( i==0 )
			{
				if(fsw0==1) //add
					Port.itfGroup=group;
				else if( (fsw0==0) && ( gsw0==group ) ) //delete
					Port.itfGroup=def_group;
			}else if( i==1 )
			{
				if(fsw1==1) //add
					Port.itfGroup=group;
				else if( (fsw1==0) && ( gsw1==group ) ) //delete
					Port.itfGroup=def_group;
			}else if( i==2 )
			{
				if(fsw2==1) //add
					Port.itfGroup=group;
				else if( (fsw2==0) && ( gsw2==group ) ) //delete
					Port.itfGroup=def_group;
			}else if( i==3 )
			{
				if(fsw3==1) //add
					Port.itfGroup=group;
				else if( (fsw3==0) && ( gsw3==group ) ) //delete
					Port.itfGroup=def_group;
			}

			mib_chain_update( MIB_SW_PORT_TBL, (unsigned char*)&Port, i );
		}
	}

check_mib:
	{
#ifdef WLAN_SUPPORT
			mib_get(MIB_WLAN_ITF_GROUP, (void *)&gwlan0);
#ifdef WLAN_MBSSID
			mib_get(MIB_WLAN_VAP0_ITF_GROUP, (void *)&gwlan1);
			mib_get(MIB_WLAN_VAP1_ITF_GROUP, (void *)&gwlan2);
			mib_get(MIB_WLAN_VAP2_ITF_GROUP, (void *)&gwlan3);
			mib_get(MIB_WLAN_VAP3_ITF_GROUP, (void *)&gwlan4);
#endif //WLAN_MBSSID		
#endif //WLAN_SUPPORT
			if( mib_chain_get(MIB_SW_PORT_TBL, 0, (void *)&Port) )
				gsw0=Port.itfGroup;
			if( mib_chain_get(MIB_SW_PORT_TBL, 1, (void *)&Port) )
				gsw1=Port.itfGroup;
			if( mib_chain_get(MIB_SW_PORT_TBL, 2, (void *)&Port) )
				gsw2=Port.itfGroup;
			if( mib_chain_get(MIB_SW_PORT_TBL, 3, (void *)&Port) )
				gsw3=Port.itfGroup;
	}
	if(
#ifdef WLAN_SUPPORT
		gwlan0==0&&
#ifdef WLAN_MBSSID
		gwlan1==0&&gwlan2==0&&gwlan3==0&&gwlan4==0&&
#endif
#endif
		gsw0==0&&gsw1==0&&gsw2==0&&gsw3==0){
			printf("\nstop portmapping!\n");
			unsigned char mode=0;
			mib_get(MIB_MPMODE, (void *)&mode);
			mode &= 0xfe;
			mib_set(MIB_MPMODE, (void *)&mode);
			pEntry->itfGroup=0;
			return 1;
	}
			
	return 1;
	
}
#endif
void formAdsl(webs_t wp, char_t *path, char_t *query)
{
	struct atmvc_entryx	entry;
	char*			pifname = NULL;
	char*			stemp = "";
	unsigned long	fstdns = 0;	//ȱʡDNS
	unsigned long	secdns = 0;	//ѡDNS
	int				ival = 0;
	int				pppnummax = 5;//default value
	int				lineno = __LINE__;

	MIB_CE_ATM_VC_T mibentry,Entry;
	int totalEntry,i;
	char tmpBuf[100];

	unsigned int ifMap;
	int drflag=0;
	char* strValue;
#ifdef CONFIG_EXT_SWITCH
	unsigned short ifgroup=0;
	int groupflag=0;
	unsigned char mode;
#endif
	char act[10];

	_TRACE_CALL;

	FETCH_INVALID_OPT(stemp, "action", _NEED);
	strncpy(act,stemp,10);
	
	if(strcmp(stemp, "rm") == 0)	//remove
	{
		int idx=-1;
		char webwanname[MAX_NAME_LEN];
		char mibwanname[MAX_NAME_LEN];
	
		_ENTRY_INT(vpi, _NEED);
		_ENTRY_INT(vci, _NEED);

		/************Place your code here, do what you want to do! ************/
		/*use 'stemp' as 'link name' to match 'atmvc_entry' entry and remove relevant entry from MIB */
		/************Place your code here, do what you want to do! ************/

		FETCH_INVALID_OPT(stemp, "lst", _NEED);
		strncpy(webwanname,stemp,MAX_NAME_LEN-1);

		// Mason Yu 
		deleteAllConnection();

		totalEntry = mib_chain_total(MIB_ATM_VC_TBL);
		for(i=0;i<totalEntry;i++){
			if(mib_chain_get(MIB_ATM_VC_TBL,i,&Entry)!=1)
				continue;
			getWanName(&Entry,mibwanname);
			if(Entry.vpi==entry.vpi&&Entry.vci==entry.vci&&(!strcmp(mibwanname,webwanname))){
				idx=i;
				break;
			}	
		}

		if(idx!=-1){

			resolveServiceDependency(idx);
			
#ifdef DNS_BIND_PVC_SUPPORT
			MIB_CE_ATM_VC_T dnsPvcEntry;
			if(mib_chain_get(MIB_ATM_VC_TBL,idx,&dnsPvcEntry)&&(dnsPvcEntry.cmode!=ADSL_BR1483))
			{
			      int tempi=0;
				unsigned char pvcifIdx=0;
				for(tempi=0;tempi<3;tempi++)
					{
					    mib_get(MIB_DNS_BIND_PVC1+tempi,(void*)&pvcifIdx);
					    if(pvcifIdx==dnsPvcEntry.ifIndex)//I get it
					    	{
					    		pvcifIdx=255;
							mib_set(MIB_DNS_BIND_PVC1+tempi,(void*)&pvcifIdx);
					    	}	
					}	
			}
#endif

//#ifdef _CWMP_MIB_ /*jiunming, for cwmp-tr069*/
			{
				MIB_CE_ATM_VC_T cwmp_entry;
				if (mib_chain_get(MIB_ATM_VC_TBL, idx, (void *)&cwmp_entry))
				{
					delPortForwarding( cwmp_entry.ifIndex );
					//delRoutingTable( cwmp_entry.ifIndex );
					delRipTable(cwmp_entry.ifIndex);
					delIpQosQueue(cwmp_entry.ifIndex);
					delIpQosRule(cwmp_entry.ifIndex);
					delIpQosTrafficRule(cwmp_entry.ifIndex);
					delBrgMacFilterRule(cwmp_entry.ifIndex);
					delIpRouteTbl(cwmp_entry.ifIndex);
				#ifdef CONFIG_USER_DDNS
					delDDNSinter(cwmp_entry.ifIndex);
				#endif
				}
			}
//#endif

#ifdef CONFIG_EXT_SWITCH
	//		groupflag=setgroupbyweb(ifgroup,"modify",&mibentry);
#endif

			if(mib_chain_delete(MIB_ATM_VC_TBL, idx) != 1) {
					strcpy(tmpBuf, T(strDelChainerror));
					goto setErr_filter;
			}

//ql add: check if it is necessary to delete a group of interface
#if 1
			{
				int wanPortNum;
				unsigned int swNum, vcNum;
				MIB_CE_SW_PORT_T Entry;
				MIB_CE_ATM_VC_T pvcEntry;
				int j, grpnum;
				char mygroup;
                           int enable_portmap =0;
						   
				vcNum = mib_chain_total(MIB_ATM_VC_TBL);
				for (grpnum=1; grpnum<=4; grpnum++) {
					wanPortNum = 0;
					
					for (j=0; j<vcNum; j++) {
						if (!mib_chain_get(MIB_ATM_VC_TBL, j, (void *)&pvcEntry))
						{
				  			//websError(wp, 400, T("Get chain record error!\n"));
				  			printf("Get chain record error!\n");
							return;
						}
						if(pvcEntry.itfGroup!=0)
							     enable_portmap++;
						if (pvcEntry.enable == 0 || pvcEntry.itfGroup!=grpnum)
							continue;

						wanPortNum++;
						
					}

					if (0 == wanPortNum) {
						printf("delete port mapping group %d\n", grpnum);
						//release LAN ports
						swNum = mib_chain_total(MIB_SW_PORT_TBL);
						for (j=swNum; j>0; j--) {
							if (!mib_chain_get(MIB_SW_PORT_TBL, j-1, (void *)&Entry))
								return;
							if (Entry.itfGroup == grpnum) {
								Entry.itfGroup = 0;
								mib_chain_update(MIB_SW_PORT_TBL, (void *)&Entry, j-1);
							}
						}
#ifdef WLAN_SUPPORT
						//release wlan0
						mib_get(MIB_WLAN_ITF_GROUP, (void *)&mygroup);
						if (mygroup == grpnum) {
							mygroup = 0;
							mib_set(MIB_WLAN_ITF_GROUP, (void *)&mygroup);
						}
#endif
#ifdef WLAN_MBSSID
						//release MBSSID
						for (j=1; j<5; j++) {
							mib_get(MIB_WLAN_VAP0_ITF_GROUP+j-1, (void *)&mygroup);
							if (mygroup == grpnum) {
								mygroup = 0;
								mib_set(MIB_WLAN_VAP0_ITF_GROUP+j-1, (void *)&mygroup);
							}
						}
#endif
						setgroup("", grpnum, lowPrio);
                                        
						break;
					}
				}

				if(!enable_portmap)
				{
				    printf("\nstop portmapping!\n");
			 
			            mib_get(MIB_MPMODE, (void *)&mode);
			             mode &= 0xfe;
			            mib_set(MIB_MPMODE, (void *)&mode);
				}
				groupflag =1;
			}//end
#endif

		}else
		{
			strcpy(tmpBuf, T(strSelectvc));
			goto setErr_filter;
		}
	         
		goto setOk_filter;
		
	}
	else if(strcmp(stemp, "sv") == 0)	//add or modify
	{
		memset(&entry, 0, sizeof(entry));
		memset(&mibentry, 0, sizeof(mibentry));

		//_GET_PSTR(ifname, _OPT);	
		//(fstdns, _OPT);
		//_GET_IP(secdns, _OPT);

		_ENTRY_INT(svtype, _NEED);
		if(entry.svtype > 4){lineno = __LINE__; goto check_err;}

		_ENTRY_INT(vpi, _NEED);
		
		_ENTRY_INT(vci, _NEED);
		if(entry.vci < 32){lineno = __LINE__; goto check_err;}
		entry.pcr=6000; //set default value
		if(entry.svtype > 0)
		{
			_ENTRY_INT(pcr, _NEED);
			if(entry.pcr < 1){lineno = __LINE__; goto check_err;}
			if(entry.svtype > 2)
			{
				_ENTRY_INT(scr, _NEED);
				if(entry.scr < 1 || entry.scr > entry.pcr){lineno = __LINE__; goto check_err;}
				_ENTRY_INT(mbs, _NEED);
				if(entry.mbs < 1){lineno = __LINE__; goto check_err;}
			}
		}

		_ENTRY_INT(encap, _NEED);
		if(entry.encap > 1){lineno = __LINE__; goto check_err;}

		_ENTRY_BOOL(napt, _NEED);

		_ENTRY_INT(cmode, _NEED);
		if(entry.cmode > 2){lineno = __LINE__; goto check_err;}

		switch(entry.cmode)
		{
		case ADSL_BR1483://bridge
			{
			}break;
		case ADSL_MER1483://route
			{
				_ENTRY_INT(ipDhcp, _NEED);
				if(entry.ipDhcp > 1){lineno = __LINE__; goto check_err;}
				if(entry.cmode == 1)//static ip
				{
					//_ENTRY_IP(ipAddr, _NEED);
					FETCH_INVALID_OPT(stemp, "ipAddr", _NEED);
					printf("\nweb ip%s\n",stemp);
					*(unsigned long*)&(entry.ipAddr) = inet_addr(stemp);
					printf("\nentry ip:%x\n",*(unsigned long*)&(entry.ipAddr));
					_ENTRY_IP(remoteIpAddr, _NEED);
					_ENTRY_IP(netMask, _NEED);
					
				}
			}break;
		case ADSL_PPPoE://pppoe
			{
				_ENTRY_STR(pppUsername, _NEED);
				_ENTRY_STR(pppPassword, _NEED);
				_ENTRY_INT(pppCtype, _NEED);
				if(entry.pppCtype > 1){lineno = __LINE__; goto check_err;}
				_ENTRY_STR(pppServiceName, _OPT);

				_ENTRY_BOOL(PPPoEProxyEnable, _NEED);
				if(entry.PPPoEProxyEnable)
				{
					_ENTRY_INT(PPPoEProxyMaxUser, _NEED);
					if(entry.PPPoEProxyMaxUser < 0 || entry.PPPoEProxyMaxUser > (unsigned)pppnummax){lineno = __LINE__; goto check_err;}
				}
				_ENTRY_BOOL(brmode, _NEED);
			}break;
		}

		_ENTRY_BOOL(vlan, _NEED);
		if(entry.vlan)
		{
			_ENTRY_INT(vid, _NEED);
			if(entry.vid > 4095){lineno = __LINE__; goto check_err;}

			_ENTRY_INT(vprio, _NEED);// entry.vprio == ()
			if(entry.vprio > 8){lineno = __LINE__; goto check_err;}
		}

		_ENTRY_INT(vpass, _OPT);

		_ENTRY_INT(itfGroup, _NEED);

		_ENTRY_BOOL(qos, _NEED);

		_ENTRY_INT(applicationtype, _NEED);
		if(entry.applicationtype > 4) {lineno = __LINE__; goto check_err;}

		_ENTRY_BOOL(dgw,_NEED);
#ifdef CONFIG_EXT_SWITCH
		if(entry.applicationtype!=2) //except tr069 connection
			ifgroup = entry.itfGroup;

#endif

		/************Place your code here, do what you want to do! ************/
		/************Place your code here, do what you want to do! ************/

		web2mib(&entry,&mibentry);
		totalEntry = mib_chain_total(MIB_ATM_VC_TBL);
		mibentry.enable=1;//always enable

		
		FETCH_INVALID_OPT(stemp, "lkname", _NEED);
		if(strcmp(stemp,"new")==0){   //add
			int cnt, pIdx;
			unsigned char vcIdx;
			int intVal;

			// Mason Yu 		
			deleteAllConnection();
			
			if (totalEntry >= MAX_VC_NUM)
			{
				strcpy(tmpBuf, T(strMaxVc));
				goto setErr_filter;
			}
			// check if connection exists
			ifMap = 0;
			cnt=0;
			
			for (i=0; i<totalEntry; i++) {
				if (!mib_chain_get(MIB_ATM_VC_TBL, i, (void *)&Entry))
				{
		  			websError(wp, 400, T(strGetChainerror));
					return;
				}
				
				if (Entry.vpi == mibentry.vpi && Entry.vci == mibentry.vci)
				{
					cnt++;
					pIdx = i;	// Jenny, for multisession PPPoE, record entry index instead of atmvc entry pointer
	//				pmyEntry = &Entry;
				}
				
#ifdef DEFAULT_GATEWAY_V1
				if (Entry.cmode != ADSL_BR1483)
					if (Entry.dgw)
						drflag = 1;
#endif

				ifMap |= 1 << (Entry.ifIndex & 0x0f);	// vc map
				ifMap |= (1 << 16) << ((Entry.ifIndex >> 4) & 0x0f);	// PPP map
			}

			
			if (cnt == 0)	// pvc not exists
			{
			
				mibentry.ifIndex = if_find_index(mibentry.cmode, ifMap);
				if (mibentry.ifIndex == 0xff)
				{
					strcpy(tmpBuf, T(strMaxVc));
					goto setErr_filter;
				}
				else if (mibentry.ifIndex == 0xef)
				{
					strcpy(tmpBuf, T(strMaxNumPPPoE));
					goto setErr_filter;
				}

#ifdef _CWMP_MIB_ /*jiunming, for cwmp-tr069*/
				mibentry.ConDevInstNum = 1 + findMaxConDevInstNum();
				if( (mibentry.cmode==ADSL_PPPoE) || 
#ifdef PPPOE_PASSTHROUGH
				    ((mibentry.cmode==ADSL_BR1483)&&(mibentry.brmode==BRIDGE_PPPOE)) ||
#endif
				    (mibentry.cmode==ADSL_PPPoA) )
					mibentry.ConPPPInstNum = 1;
				else
					mibentry.ConIPInstNum = 1;
				//fprintf( stderr, "<%s:%d>NewInstNum=>ConDev:%u, PPPCon:%u, IPCon:%u\n", __FILE__, __LINE__, mibentry.ConDevInstNum, mibentry.ConPPPInstNum, mibentry.ConIPInstNum );
#endif
			}else{
				if (!mib_chain_get(MIB_ATM_VC_TBL, pIdx, (void *)&Entry)) {	// Jenny, for multisession PPPoE, get existed pvc config
					strcpy(tmpBuf, errGetEntry);
					goto setErr_filter;
				}
				if (Entry.cmode == ADSL_PPPoE && mibentry.cmode == ADSL_PPPoE)
				{
					if (cnt<MAX_POE_PER_VC)
					{	// get the pvc info.
						mibentry.qos = Entry.qos;
						mibentry.pcr = Entry.pcr;
						mibentry.encap = Entry.encap;
						ifMap &= 0xffff0000; // don't care the vc part
						mibentry.ifIndex = if_find_index(mibentry.cmode, ifMap);
						if (mibentry.ifIndex == 0xef)
						{
							strcpy(tmpBuf, T(strMaxNumPPPoE));
							goto setErr_filter;
						}
						mibentry.ifIndex &= 0xf0;
						mibentry.ifIndex |= (Entry.ifIndex&0x0f);
#ifdef PPPOE_PASSTHROUGH
						mibentry.brmode = Entry.brmode;	// Jenny, for multisession PPPoE
#endif

#ifdef _CWMP_MIB_ /*jiunming, for cwmp-tr069*/
						mibentry.ConDevInstNum = Entry.ConDevInstNum;
						mibentry.ConPPPInstNum = 1 + findMaxPPPConInstNum( Entry.ConDevInstNum );
						//fprintf( stderr, "<%s:%d>NewInstNum=>ConDev:%u, PPPCon:%u, IPCon:%u\n", __FILE__, __LINE__, mibentry.ConDevInstNum, mibentry.ConPPPInstNum, mibentry.ConIPInstNum );
#endif
					}
					else
					{
						strcpy(tmpBuf, T(strMaxNumPPPoE));
						goto setErr_filter;
					}
				}
				else
				{
					strcpy(tmpBuf, T(strConnectExist));
					goto setErr_filter;
				}
			}		
#ifdef DEFAULT_GATEWAY_V1
			if (mibentry.cmode != ADSL_BR1483)
				if (drflag && mibentry.dgw)
				{
					strcpy(tmpBuf, T(strDrouteExist));
					goto setErr_filter;
				}
#endif
/* 
#ifdef CONFIG_ATM_CLIP
			if (mibentry.cmode == ADSL_RT1577)
				mibentry.encap = 1;	// LLC
#endif
*///set by web
			// set default
	//		mibentry.dgw = 1;
			if (mibentry.cmode == ADSL_PPPoE)
			{
				mibentry.mtu = 1492;
/*
#ifdef CONFIG_USER_PPPOE_PROXY
				mibentry.PPPoEProxyMaxUser=4;
				mibentry.PPPoEProxyEnable=0;
#endif
*///set by web
			}
			else
				mibentry.mtu = 1500;
/*	
#ifdef CONFIG_EXT_SWITCH
			// VLAN
			mibentry.vlan = 0;	// disable
			mibentry.vid = 0; // VLAN tag
			mibentry.vprio = 0; // priority bits (0 ~ 7)
			mibentry.vpass = 0; // no pass-through
#endif
*///set by web
#ifdef CONFIG_EXT_SWITCH
			groupflag=setgroupbyweb(ifgroup,"add",&mibentry);
#endif
// Mason Yu 

/*			if(mib_chain_add(MIB_ATM_VC_TBL, (unsigned char*)&mibentry) != 1){
				strcpy(tmpBuf, T(strAddChainerror));
				goto setErr_filter;
			}
*/
			intVal = mib_chain_add(MIB_ATM_VC_TBL, (unsigned char*)&mibentry);		
			if (intVal == 0) {			
				strcpy(tmpBuf, T(strAddChainerror));			
				goto setErr_filter;		
			}
			else if (intVal == -1) {			
				strcpy(tmpBuf, T(strTableFull));			
				goto setErr_filter;		
			}

			//ql_xu: add qos queue 
			if (mibentry.qosenable)
				addIpQosQueue(mibentry.ifIndex);			
		
			goto setOk_filter;

		}else{   //modify
			int cnt=0, pIdx;	
			int selected=-1;
			MIB_CE_ATM_VC_T myEntry;

			char webwanname[MAX_NAME_LEN];
			char mibwanname[MAX_NAME_LEN];

			ifMap=0;
			FETCH_INVALID_OPT(stemp, "lst", _NEED);
			strncpy(webwanname,stemp,MAX_NAME_LEN-1);

			// Mason Yu 		
			deleteAllConnection();

			for (i=0; i<totalEntry; i++) {
				if (!mib_chain_get(MIB_ATM_VC_TBL, i, (void *)&Entry))
				{
		  			websError(wp, 400, T(strGetChainerror));
					return;
				}

				getWanName(&Entry,mibwanname);
				printf("\nmibname=%s\n",mibwanname);
		DEBUGPRRINT;
				if (Entry.vpi == mibentry.vpi && Entry.vci == mibentry.vci ) 
					cnt++;
		DEBUGPRRINT;
				if ( (selected == -1) && !strcmp(webwanname,mibwanname) )
					selected = i;
				else
				{
		DEBUGPRRINT;
					ifMap |= 1 << (Entry.ifIndex & 0x0f);	// vc map
					ifMap |= (1 << 16) << ((Entry.ifIndex >> 4) & 0x0f);	// PPP map
				}
	        DEBUGPRRINT;
#ifdef DEFAULT_GATEWAY_V1
				if (Entry.cmode != ADSL_BR1483)
					if (Entry.dgw)
						drflag = 1;
#endif
			}

			if(cnt > 0) {
			//Make sure there is no mismatch mode
				for (i=0, cnt=0; i<totalEntry; i++) {
		DEBUGPRRINT;
					if(i==selected)
						continue;
					if (!mib_chain_get(MIB_ATM_VC_TBL, i, (void *)&Entry)){
		  				websError(wp, 400, T(strGetChainerror));
						return;
					}
		DEBUGPRRINT;
					if (Entry.vpi == mibentry.vpi && Entry.vci == mibentry.vci){
						cnt++;
		DEBUGPRRINT;
						if(Entry.cmode != mibentry.cmode){
							strcpy(tmpBuf, T(strConnectExist));
							goto setErr_filter;											
						}
		DEBUGPRRINT;
						if (mibentry.cmode == ADSL_PPPoE)		// Jenny, for multisession PPPoE support
							pIdx = i;
					}
				}
		DEBUGPRRINT;
				//Max. 2 PPPoE connections
	//			if(entry.cmode == ADSL_PPPoE && cnt==2) {
				if(mibentry.cmode == ADSL_PPPoE && cnt==MAX_POE_PER_VC) {	// Jenny, multisession PPPoE support
					strcpy(tmpBuf, T(strMaxNumPPPoE));
					goto setErr_filter;		
				//Max. 1 connection except PPPoE
				} else if(mibentry.cmode != ADSL_PPPoE&& cnt>0 ) {
					strcpy(tmpBuf, T(strConnectExist));
					goto setErr_filter;
				}
				if (mibentry.cmode == ADSL_PPPoE && cnt>0)		// Jenny, for multisession PPPoE, get existed PPPoE config for further ifindex use
					if (!mib_chain_get(MIB_ATM_VC_TBL, pIdx, (void *)&myEntry)) {
						strcpy(tmpBuf, errGetEntry);
						goto setErr_filter;
					}
			}
		DEBUGPRRINT;
			if (!mib_chain_get(MIB_ATM_VC_TBL, selected, (void *)&Entry)) {
				strcpy(tmpBuf, errGetEntry);
				goto setErr_filter;
			}

			// restore stuff not posted in this form
			if (mibentry.cmode == ADSL_PPPoE)
				if (cnt > 0) {		// Jenny, for multisession PPPoE, ifIndex(VC device) must refer to existed PPPoE connection
					ifMap &= 0xffff0000; // don't care the vc part
					mibentry.ifIndex = if_find_index(mibentry.cmode, ifMap);
					mibentry.ifIndex &= 0xf0;
					mibentry.ifIndex |= (myEntry.ifIndex&0x0f);
				}
				else {
		DEBUGPRRINT;
					mibentry.ifIndex = if_find_index(mibentry.cmode, ifMap);
/*
#ifdef PPPOE_PASSTHROUGH
					if (mibentry.cmode == Entry.cmode)
						mibentry.brmode = Entry.brmode;
#endif
*///set by web
				}
			else
				mibentry.ifIndex = Entry.ifIndex;
	DEBUGPRRINT;
/*
			entry.qos = Entry.qos;
			entry.pcr = Entry.pcr;
			entry.scr = Entry.scr;
			entry.mbs = Entry.mbs;
			entry.cdvt = Entry.cdvt;
*///set by web			
			entry.pppAuth = Entry.pppAuth;
			mibentry.rip = Entry.rip;
	//		entry.dgw = Entry.dgw;
			mibentry.mtu = Entry.mtu;

#ifdef CONFIG_EXT_SWITCH
			//ql: when pvc is modified, interface group don't changed???
			mibentry.itfGroup = Entry.itfGroup;
#endif

	DEBUGPRRINT;
#ifdef CONFIG_SPPPD_STATICIP
	                if(mibentry.cmode == ADSL_PPPoE)
			  {
			    mibentry.pppIp = Entry.pppIp;
			    strcpy( mibentry.ipAddr, Entry.ipAddr);
	                  }
#endif
	DEBUGPRRINT;
/*
#ifdef PPPOE_PASSTHROUGH
			if (entry.cmode != ADSL_PPPoE)
				if (entry.cmode == Entry.cmode)
					entry.brmode = Entry.brmode;
#endif
*///set by web
#ifdef CONFIG_EXT_SWITCH
/*
			// VLAN
			entry.vlan = Entry.vlan;
			entry.vid = Entry.vid;
			entry.vprio = Entry.vprio;
			entry.vpass = Entry.vpass;
*///set by web
#ifdef _PRMT_X_CT_COM_ETHLINK_ 
			//mibentry.pmark = Entry.pmark;
#endif
#endif
	DEBUGPRRINT;
#ifdef _CWMP_MIB_ /*jiunming, for cwmp-tr069*/
			mibentry.connDisable = 0;
			if (Entry.vpi == mibentry.vpi && Entry.vci == mibentry.vci)
			{
				mibentry.ConDevInstNum = Entry.ConDevInstNum;
				if( (mibentry.cmode==ADSL_PPPoE) || 
#ifdef PPPOE_PASSTHROUGH
				    ((mibentry.cmode==ADSL_BR1483)&&(mibentry.brmode==BRIDGE_PPPOE)) ||
#endif
				    (mibentry.cmode==ADSL_PPPoA) )
				{
					if( Entry.ConPPPInstNum!=0 )
						mibentry.ConPPPInstNum = Entry.ConPPPInstNum;
					else
						mibentry.ConPPPInstNum = 1 + findMaxPPPConInstNum( mibentry.ConDevInstNum );
					mibentry.ConIPInstNum = 0;
				}else{
					mibentry.ConPPPInstNum = 0;
					if(Entry.ConIPInstNum!=0)
						mibentry.ConIPInstNum = Entry.ConIPInstNum;
					else
						mibentry.ConIPInstNum = 1 + findMaxIPConInstNum( mibentry.ConDevInstNum );
				}
			}else{
	DEBUGPRRINT;
				unsigned int  instnum=0;
				instnum = findConDevInstNumByPVC( mibentry.vpi, mibentry.vci );
				if(instnum==0)
					instnum = 1 + findMaxConDevInstNum();
				mibentry.ConDevInstNum = instnum;
				if( (mibentry.cmode==ADSL_PPPoE) || 
#ifdef PPPOE_PASSTHROUGH
				    ((mibentry.cmode==ADSL_BR1483)&&(mibentry.brmode==BRIDGE_PPPOE)) ||
#endif
				    (mibentry.cmode==ADSL_PPPoA) )
				{
					mibentry.ConPPPInstNum = 1 + findMaxPPPConInstNum( mibentry.ConDevInstNum );
					mibentry.ConIPInstNum = 0;
				}else{
					mibentry.ConPPPInstNum = 0;
					mibentry.ConIPInstNum = 1 + findMaxIPConInstNum( mibentry.ConDevInstNum );
				}
	DEBUGPRRINT;
			}
			//fprintf( stderr, "<%s:%d>NewInstNum=>ConDev:%u, PPPCon:%u, IPCon:%u\n", __FILE__, __LINE__, mibentry.ConDevInstNum, mibentry.ConPPPInstNum, mibentry.ConIPInstNum );

			mibentry.autoDisTime = Entry.autoDisTime;
			mibentry.warnDisDelay = Entry.warnDisDelay;
			//strcpy( entry.pppServiceName, Entry.pppServiceName );
			strcpy( mibentry.WanName, Entry.WanName );
/*
#ifdef _PRMT_X_CT_COM_PPPOEv2_
			entry.PPPoEProxyEnable = Entry.PPPoEProxyEnable;
			entry.PPPoEProxyMaxUser = Entry.PPPoEProxyMaxUser;
#endif //_PRMT_X_CT_COM_PPPOEv2_
*///set by web
#ifdef _PRMT_X_CT_COM_WANEXT_
//			mibentry.ServiceList = Entry.ServiceList;
#endif //_PRMT_X_CT_COM_WANEXT_
#endif //_CWMP_MIB_
			strcpy(mibentry.pppACName, Entry.pppACName);

			if (mibentry.enable == 0 && Entry.enable == 1)
				resolveServiceDependency(selected);

			
#ifdef DEFAULT_GATEWAY_V1
			// set default route flag
			if (mibentry.cmode != ADSL_BR1483)
				if (!mibentry.dgw && Entry.dgw)
					drflag =0;
			else
				if (Entry.dgw)
					drflag = 0;
#endif
	DEBUGPRRINT;
			// find the ifIndex
			if (mibentry.cmode != Entry.cmode)
			{
				if (!(mibentry.cmode == ADSL_PPPoE && cnt>0))	// Jenny, entries except multisession PPPoE
					mibentry.ifIndex = if_find_index(mibentry.cmode, ifMap);
				if (mibentry.ifIndex == 0xff)
				{
					strcpy(tmpBuf, T(strMaxVc));
					goto setErr_filter;
				}
				else if (mibentry.ifIndex == 0xef)
				{
					strcpy(tmpBuf, T(strMaxNumPPPoE));
					goto setErr_filter;
				}
	DEBUGPRRINT;			
				// mode changed, restore to default
				if (mibentry.cmode == ADSL_PPPoE) {
					mibentry.mtu = 1492;
/*
#ifdef CONFIG_USER_PPPOE_PROXY
					entry.PPPoEProxyMaxUser=4;
#endif
					entry.pppAuth = 0;
*///set by web
				}
				else {
/*
#ifdef CONFIG_USER_PPPOE_PROXY
					entry.PPPoEProxyMaxUser=0;
#endif
*/
	//				entry.dgw = 1;
					mibentry.mtu = 1500;
				}
	DEBUGPRRINT;			
	//			entry.dgw = 1;
/*
#ifdef CONFIG_EXT_SWITCH
				// VLAN
				entry.vlan = 0;	// disable
				entry.vid = 0; // VLAN tag
				entry.vprio = 0; // priority bits (0 ~ 7)
				entry.vpass = 0; // no pass-through
#endif
*///set by web
			}

#ifdef DEFAULT_GATEWAY_V1
			if (mibentry.cmode != ADSL_BR1483)
			{
				if (drflag && mibentry.dgw && !Entry.dgw)
				{
					strcpy(tmpBuf, T(strDrouteExist));
					goto setErr_filter;
				}
				if (mibentry.dgw && !Entry.dgw)
					drflag = 1;
			}
#endif

DEBUGPRRINT;
			if( mibentry.ifIndex!=Entry.ifIndex)
			{
				updatePortForwarding( Entry.ifIndex, mibentry.ifIndex );
				//updateRoutingTable( Entry.ifIndex, mibentry.ifIndex );
				delRipTable(Entry.ifIndex);
				delIpQosQueue(Entry.ifIndex);
				delIpQosRule(Entry.ifIndex);
				delIpQosTrafficRule(Entry.ifIndex);
		#ifdef CONFIG_USER_DDNS
				delDDNSinter(Entry.ifIndex);
		#endif
				delBrgMacFilterRule(Entry.ifIndex);
				delIpRouteTbl(Entry.ifIndex);
			}
			else if (0 == mibentry.qosenable)
			{
				delIpQosQueue(mibentry.ifIndex);
				delIpQosRule(mibentry.ifIndex);
				delIpQosTrafficRule(mibentry.ifIndex);
			}
			else if (mibentry.cmode != ADSL_BR1483)
			{
				delBrgMacFilterRule(mibentry.ifIndex);
			}
DEBUGPRRINT;

#ifdef CONFIG_EXT_SWITCH
			groupflag=setgroupbyweb(ifgroup,"modify",&mibentry);
#endif
/*
#ifdef CONFIG_ATM_CLIP
			if (entry.cmode == ADSL_RT1577)
				entry.encap = 1;	// LLC
#endif
*/

	DEBUGPRRINT;
			// log message
	//add by ramen for ZTE ROTUER rmt acc on pvc


	//add by ramen for DNS bind pvc
#ifdef DNS_BIND_PVC_SUPPORT
	MIB_CE_ATM_VC_T dnsPvcEntry;
	if(mib_chain_get(MIB_ATM_VC_TBL,selected,&dnsPvcEntry)&&(dnsPvcEntry.cmode!=ADSL_BR1483))
	{
	      int tempi=0;
		unsigned char pvcifIdx=0;
		for(tempi=0;tempi<3;tempi++)
			{
			    mib_get(MIB_DNS_BIND_PVC1+tempi,(void*)&pvcifIdx);
			    if(pvcifIdx==dnsPvcEntry.ifIndex)//I get it
			    	{
			    	  if(mibentry.cmode==ADSL_BR1483)	 		pvcifIdx=255;				
			    	  	else    	  	pvcifIdx=mibentry.ifIndex;		    	  		
					mib_set(MIB_DNS_BIND_PVC1+tempi,(void*)&pvcifIdx);
			    	}			
			}
		
	}
#endif
	        //jim garbage action...
			//memcpy(&Entry, &entry, sizeof(entry));
			// log message
			// Mason Yu 

/*
	//add by ramen to replace the default gateway
#if defined(ZTE_GENERAL_ROUTER_SC)||defined(ZTE_GENERAL_ROUTER_EN)
	strValue = websGetVar(wp, T("modifydgw"), T(""));
	printf("strValue=%s\n",strValue);
	if(!strcmp(strValue,"enable")){
		
	strValue = websGetVar(wp, T("pastdgw"), T(""));
		    printf("pastdgw strValue=%s\n",strValue);
		    unsigned char  pastdgw=strValue[0]-'0';
		   if(!mib_chain_get(MIB_ATM_VC_TBL, pastdgw, (void *)&Entry)){
		   	websError(wp, 400, T(strGetChainerror));
			return;
		   	}
		   Entry.dgw=0;
		   mib_chain_update(MIB_ATM_VC_TBL,(void*)&Entry,pastdgw);
	}
	DEBUGPRRINT;
#endif
*/
			mib_chain_update(MIB_ATM_VC_TBL, (void *)&mibentry, selected);
			//ql_xu: add IP QoS queue
			if (mibentry.qosenable)
				addIpQosQueue(mibentry.ifIndex);

			restart_dnsrelay(); //star
			DEBUGPRRINT;
			
//jim moved here from line 2055 for do connection after mib updated...
#if 0
#ifdef ZTE_531B_BRIDGE_SC
			unsigned char vChar;
#ifdef __CLOSE_INTERFACE_BEFORE_WRITE_
			/*to avoid the dead when wrriting the flash*/
			itfcfg("sar", 0);
			itfcfg("eth0", 0);
#ifdef WLAN_SUPPORT
			itfcfg("wlan0", 0);
#endif
#endif /*__CLOSE_INTERFACE_BEFORE_WRITE_*/
			mib_update(CURRENT_SETTING, CONFIG_MIB_ALL);
	DEBUGPRRINT;
#ifdef __CLOSE_INTERFACE_BEFORE_WRITE_
			itfcfg("sar", 1);
			 itfcfg("eth0", 1);
#ifdef WLAN_SUPPORT
			vChar=0;
			mib_get( MIB_WLAN_DISABLED, (void *)&vChar);
			if( vChar==0 )	itfcfg("wlan0", 1);
#endif //WLAN_SUPPORT
#endif /*__CLOSE_INTERFACE_BEFORE_WRITE_*/
#endif
#endif
			goto setOk_filter;		

		}
	}
	else {lineno = __LINE__; goto check_err;}
	
	_COND_REDIRECT;
check_err:
	_TRACE_LEAVEL;
	strcpy(tmpBuf, T(""));
	goto setErr_filter;
	return;
	
setOk_filter:

#ifdef WRITE_TO_FLASH
	mib_update(CURRENT_SETTING, CONFIG_MIB_ALL);
#endif

//add by ramen to take effect rip
	startRip();
DEBUGPRRINT;
	// Added by Mason Yu. for take effect in real time	
	RegisterPVCnumber();	
	DEBUGPRRINT;
	cleanAllFirewallRule();
	DEBUGPRRINT;
	restart_dnsrelay(); //Jenny
	DEBUGPRRINT;
	startWan(BOOT_LAST);
	DEBUGPRRINT;//ql_xu
	take_qos_effect();

#ifdef CONFIG_EXT_SWITCH
	printf("\n*******************groupflag=%d\n",groupflag);
	if(groupflag==1){
		
		mib_get(MIB_MPMODE, (void *)&mode);   	
		if (mode&0x01){
			printf("**************setupEth2pvc************\n");
			__dev_register_swport();
			setupEth2pvc();
		}
		else if (strcmp(act, "rm") == 0){
                    setupEth2pvc();
		}
		
	}
#endif
/*
#ifdef _CWMP_MIB_
	off_tr069();
	if (-1==startCWMP()){
		printf("tr069ͻ˳ʧ!");
	}
#endif	
*/
	REDIRECTSUBMITURL;
	return;

setErr_filter:
	ERR_MSG(tmpBuf);

	RegisterPVCnumber();		
	cleanAllFirewallRule();
	restart_dnsrelay(); //Jenny
	startWan(BOOT_LAST);
}

void formAdslClt(webs_t wp, char_t *path, char_t *query)
{
	struct atmvc_entryx	entry;
	char*			stemp = "";
	int				errcode = 1, lineno = __LINE__;

	_TRACE_CALL;

	memset(&entry, 0, sizeof(entry));

	_ENTRY_INT(vpi, _NEED);
	if(entry.vpi < 0 || entry.vpi > 255){lineno = __LINE__; goto check_err;}
	
	_ENTRY_INT(vci, _NEED);
	if(entry.vci < 32 || entry.vci > 65535){lineno = __LINE__; goto check_err;}
		
	_ENTRY_STR(pppUsername, _NEED);
	_ENTRY_STR(pppPassword, _NEED);

	MIB_CE_ATM_VC_T Entry;
	int totalEntry,i;
	int idx;
	char webwanname[MAX_NAME_LEN];
	char mibwanname[MAX_NAME_LEN];
	char tmpBuf[256];

	FETCH_INVALID_OPT(stemp, "lkname", _NEED);
	strncpy(webwanname,stemp,MAX_NAME_LEN-1);
		
	totalEntry = mib_chain_total(MIB_ATM_VC_TBL);
	for(i=0;i<totalEntry;i++){
			if(mib_chain_get(MIB_ATM_VC_TBL,i,&Entry)!=1)
				continue;
			getWanName(&Entry,mibwanname);
			if(Entry.vpi==entry.vpi&&Entry.vci==entry.vci&&(!strcmp(mibwanname,webwanname))){
				idx=i;
				break;
			}	
	}

	if(idx!=-1){
		// Mason Yu 		
		stopConnection(&Entry);
		strcpy(Entry.pppUsername,entry.pppUsername);
		strcpy(Entry.pppPassword, entry.pppPassword);
		mib_chain_update(MIB_ATM_VC_TBL, (void *)&Entry, idx);		
		startConnection(&Entry);
#ifdef WRITE_TO_FLASH
		mib_update(CURRENT_SETTING, CONFIG_MIB_ALL);
#endif
	}else
	{
		strcpy(tmpBuf, T(strSelectvc));
		goto setErr_filter;
	}

	
	errcode = 0;
	_COND_REDIRECT;
check_err:
	_TRACE_LEAVEL;
	return;

setErr_filter:	
	ERR_MSG(tmpBuf);
}

//add by chenzhuoxin
void formSetAdsl(webs_t wp, char_t *path, char_t *query)
{
	char_t *	strVal = NULL;
	short		adslMode = 0;
	int			lineno = __LINE__;
	int 		xmode;
	
	_TRACE_CALL;

	strVal = websGetVar(wp, T("glite"), T(""));
	if ( 0 == strcmp(strVal,"on"))
		adslMode |= ADSL_MODE_GLITE;
	
	strVal = websGetVar(wp, T("gdmt"), T(""));
	if ( 0 == strcmp(strVal,"on"))
		adslMode |= ADSL_MODE_GDMT;
	
	strVal = websGetVar(wp, T("t1413"), T(""));
	if ( 0 == strcmp(strVal,"on"))		
		adslMode |= ADSL_MODE_T1413;
	
	strVal = websGetVar(wp, T("adsl2"), T(""));
	if ( 0 == strcmp(strVal,"on"))
		adslMode |= ADSL_MODE_ADSL2;
	
	strVal = websGetVar(wp, T("anxl"), T(""));
	if ( 0 == strcmp(strVal,"on"))		
		adslMode |= ADSL_MODE_ANXL;
	
	strVal = websGetVar(wp, T("adsl2p"), T(""));
	if ( 0 == strcmp(strVal,"on"))
		adslMode |= ADSL_MODE_ADSL2P;
	
	strVal = websGetVar(wp, T("anxm"), T(""));
	if ( 0 == strcmp(strVal,"on"))		
		adslMode |= ADSL_MODE_ANXM;

	printf("adslmode =%x\n",adslMode);

	/*
	 *add code by RTL
       */
       	if (!mib_set(MIB_ADSL_MODE, (void *)&adslMode)) {
		printf("Set ADSL mode error!");
		return;
	}

	mib_update(CURRENT_SETTING, CONFIG_MIB_ALL);

	//apply change
	xmode=0;
	if (adslMode & (ADSL_MODE_GLITE|ADSL_MODE_T1413|ADSL_MODE_GDMT))
		xmode |= 1;	// ADSL1
	if (adslMode & ADSL_MODE_ADSL2)
		xmode |= 2;	// ADSL2
	if (adslMode & ADSL_MODE_ADSL2P)
		xmode |= 4;	// ADSL2+
	adsl_drv_get(RLCM_SET_XDSL_MODE, (void *)&xmode, 4);

	xmode = adslMode & (ADSL_MODE_GLITE|ADSL_MODE_T1413|ADSL_MODE_GDMT);	//  1: ansi, 2: g.dmt, 8:g.lite
	adsl_drv_get(RLCM_SET_ADSL_MODE, (void *)&xmode, 4);	
	
	if (adslMode & ADSL_MODE_ANXL)	// Annex L
		xmode = 3; // Wide-Band & Narrow-Band Mode
	else
		xmode = 0;
	adsl_drv_get(RLCM_SET_ANNEX_L, (void *)&xmode, 4);
	
	if (adslMode & ADSL_MODE_ANXM)	// Annex M
		xmode = 1;
	else
		xmode = 0;
	adsl_drv_get(RLCM_SET_ANNEX_M, (void *)&xmode, 4);

	adsl_drv_get(RLCM_MODEM_RETRAIN, NULL, 0);

	_COND_REDIRECT;
check_err:
	_TRACE_LEAVEL;
	return;
}

//ADSL init function
int init_dsl_page(int eid, webs_t wp, int argc, char_t **argv)
{
	short	adslMode = 0;
	int		lineno = __LINE__;
	
	_TRACE_CALL;

	/*test code*/
	adslMode = 0xfb;

	/*
	 *Add code by Realtek
	 */

	websWrite(wp, "glite_sw = %s;\n", (adslMode & ADSL_MODE_GLITE) ? "true" : "false");
	
	websWrite(wp, "gdmt_sw = %s;\n", (adslMode & ADSL_MODE_GDMT) ? "true" : "false");
	
	websWrite(wp, "t1413_sw = %s;\n", (adslMode & ADSL_MODE_T1413) ? "true" : "false");
	
	websWrite(wp, "adsl2_sw = %s;\n", (adslMode & ADSL_MODE_ADSL2) ? "true" : "false");
	
	websWrite(wp, "anxl_sw= %s;\n", (adslMode & ADSL_MODE_ANXL) ? "true" : "false");
	
	websWrite(wp, "adsl2p_sw = %s;\n", (adslMode & ADSL_MODE_ADSL2P) ? "true" : "false");
	
	websWrite(wp, "anxm_sw= %s;\n", (adslMode & ADSL_MODE_ANXM) ? "true" : "false");

	/*
	 *Add code by Realtek
	 */
	
check_err:
	_TRACE_LEAVEL;
	return 0;
}


//interface status
extern int enetStatus(int port);
int getifstatus(int eid, webs_t wp, int argc, char_t **argv)
{
	unsigned char	power = 1;	//0 - down, 1 - up
	unsigned char	dsl = 0;	//0 - down, 1 - up
	unsigned char	wan = 0;	//0 - down, 1 - up
	unsigned char	lan1 = 0;	//0 - down, 1 - up
	unsigned char	iTV = 0;	//0 - down, 1 - up
	unsigned char	lan3 = 0;	//0 - down, 1 - up
	unsigned char	lan4 = 0;	//0 - down, 1 - up
	unsigned char	wlan = 0;	//0 - down, 1 - up
	unsigned char	wps = 0;	//0 - down, 1 - up
	unsigned char	usb = 0;	//0 - down, 1 - up
	int		errcode = 1, lineno = __LINE__;
	unsigned char no_wlan;
	unsigned char buffer[256];
	int tmp;
	char MNT_FILE[] = "/tmp/usb/mnt_map";
	FILE * fp;
	_TRACE_CALL;

//dsl
	getAdslInfo(ADSL_GET_STATE, buffer, sizeof(buffer));
	if(!strncmp(buffer, "SHOWTIME", 8))
		dsl=1;
	else
		dsl=0;

//wan
	tmp=Internet_status(1);
	wan=(unsigned char)(tmp);
	
//lan
	lan1=enetStatus(1);

	iTV=enetStatus(2);

	lan3=enetStatus(3);

	lan4=enetStatus(4);	

//wlan
#ifdef WLAN_SUPPORT
	mib_get(MIB_WLAN_DISABLED, (void *)&no_wlan);
	if(no_wlan==1)
		wlan=0;
	else
		wlan=1;
#endif

//usb state
	fp = fopen(MNT_FILE, "r");
	if (!fp) {
		usb=0;
	} else {
		fclose(fp);
		usb=1;
	}
	_PUT_BOOL(power);
	_PUT_BOOL(dsl);
	_PUT_BOOL(wan);
	_PUT_BOOL(lan1);
	_PUT_BOOL(iTV);
	_PUT_BOOL(lan3);
	_PUT_BOOL(lan4);
	_PUT_BOOL(wlan);
	_PUT_BOOL(wps);
	_PUT_BOOL(usb);

check_err:
	_TRACE_LEAVEL;
	return 0;
}

